@@ -755,29 +755,45 @@ class CarModel:
         ]
 
         # Year index for EURO 6 pollution class, starts in 2015
-
         self.array.loc[
             :, ["ICEV-d", "PHEV-c-d", "HEV-d"], list_direct_emissions, [y for y in self.array.year.values if y>=2015]
         ] = hem.get_emissions_per_powertrain("diesel", euro_class=6)
 
-        # Year index for EURO 5 pollution class, starts in 2009
+        # Year index for EURO 5 pollution class, starts in 2011, ends in 2014
         self.array.loc[
             :, ["ICEV-d", "PHEV-c-d", "HEV-d"], list_direct_emissions, [y for y in self.array.year.values if
-                                                                        2009 <= y < 2015]
+                                                                        2011 <= y < 2015]
         ] = hem.get_emissions_per_powertrain("diesel", euro_class=5)
 
-        # Year index for EURO 4 pollution class, starts in 2005, ends in 2008
+        # Year index for EURO 4 pollution class, starts in 2006, ends in 2010
         self.array.loc[
             :, ["ICEV-d", "PHEV-c-d", "HEV-d"], list_direct_emissions, [y for y in self.array.year.values if
-                                                                        2005 <= y < 2009]
+                                                                        2006 <= y < 2011]
         ] = hem.get_emissions_per_powertrain("diesel", euro_class=4)
 
-        # Year index for EURO 3 pollution class, starts in 2000, ends in 2004
+        # Year index for EURO 3 pollution class, starts in 2001, ends in 2005
         self.array.loc[
             :, ["ICEV-d", "PHEV-c-d", "HEV-d"], list_direct_emissions, [y for y in self.array.year.values if
-                                                                        2000 <= y < 2004]
+                                                                        2001 <= y < 2006]
         ] = hem.get_emissions_per_powertrain("diesel", euro_class=3)
 
+        # Year index for EURO 2 pollution class, starts in 1997, ends in 2000
+        self.array.loc[
+            :, ["ICEV-d", "PHEV-c-d", "HEV-d"], list_direct_emissions, [y for y in self.array.year.values if
+                                                                        1997 <= y < 2001]
+        ] = hem.get_emissions_per_powertrain("diesel", euro_class=2)
+
+        # Year index for EURO 1 pollution class, starts in 1993, ends in 1996
+        self.array.loc[
+            :, ["ICEV-d", "PHEV-c-d", "HEV-d"], list_direct_emissions, [y for y in self.array.year.values if
+                                                                        1993 <= y < 1997]
+        ] = hem.get_emissions_per_powertrain("diesel", euro_class=1)
+
+        # Year index for EURO 0 pollution class, ends in 1992
+        self.array.loc[
+            :, ["ICEV-d", "PHEV-c-d", "HEV-d"], list_direct_emissions, [y for y in self.array.year.values if y < 1993]
+        ] = hem.get_emissions_per_powertrain("diesel", euro_class=0)
+
         # Applies an emission factor, useful for sensitivity purpose
         self.array.loc[
             :, ["ICEV-d", "PHEV-c-d", "HEV-d"], list_direct_emissions, :
@@ -788,24 +804,41 @@ class CarModel:
             :, ["ICEV-p", "HEV-p", "PHEV-c-p"], list_direct_emissions, [y for y in self.array.year.values if y>=2015]
         ] = hem.get_emissions_per_powertrain("petrol", euro_class=6)
 
-        # Year index for EURO 5 pollution class, starts in 2009, ends in 2014
+        # Year index for EURO 5 pollution class, starts in 2011, ends in 2014
         self.array.loc[
             :, ["ICEV-p", "HEV-p", "PHEV-c-p"], list_direct_emissions, [y for y in self.array.year.values if
-                                                                        2009 <= y < 2015]
+                                                                        2011 <= y < 2015]
         ] = hem.get_emissions_per_powertrain("petrol", euro_class=5)
 
-        # Year index for EURO 4 pollution class, starts in 2005, ends in 2008
+        # Year index for EURO 4 pollution class, starts in 2006, ends in 2010
         self.array.loc[
             :, ["ICEV-p", "HEV-p", "PHEV-c-p"], list_direct_emissions, [y for y in self.array.year.values if
-                                                                        2005 <= y < 2009]
+                                                                        2006 <= y < 2011]
         ] = hem.get_emissions_per_powertrain("petrol", euro_class=4)
 
-        # Year index for EURO 3 pollution class, starts in 2000, ends in 2004
+        # Year index for EURO 3 pollution class, starts in 2001, ends in 2005
         self.array.loc[
             :, ["ICEV-p", "HEV-p", "PHEV-c-p"], list_direct_emissions, [y for y in self.array.year.values if
-                                                                        2000 <= y < 2004]
+                                                                        2001 <= y < 2006]
         ] = hem.get_emissions_per_powertrain("petrol", euro_class=3)
 
+        # Year index for EURO 2 pollution class, starts in 1997, ends in 2000
+        self.array.loc[
+            :, ["ICEV-p", "HEV-p", "PHEV-c-p"], list_direct_emissions, [y for y in self.array.year.values if
+                                                                        1997 <= y < 2001]
+        ] = hem.get_emissions_per_powertrain("petrol", euro_class=2)
+
+        # Year index for EURO 1 pollution class, starts in 1993, ends in 1996
+        self.array.loc[
+            :, ["ICEV-p", "HEV-p", "PHEV-c-p"], list_direct_emissions, [y for y in self.array.year.values if
+                                                                        1993 <= y < 1997]
+        ] = hem.get_emissions_per_powertrain("petrol", euro_class=1)
+
+        # Year index for EURO 0 pollution class, starts ends in 1992
+        self.array.loc[
+            :, ["ICEV-p", "HEV-p", "PHEV-c-p"], list_direct_emissions, [y for y in self.array.year.values if y < 1993]
+        ] = hem.get_emissions_per_powertrain("petrol", euro_class=0)
+
         # Applies an emission factor, useful for sensitivity purpose
         self.array.loc[
             :, ["ICEV-p", "HEV-p", "PHEV-c-p"], list_direct_emissions, :
@@ -816,21 +849,26 @@ class CarModel:
             :, "ICEV-g", list_direct_emissions, [y for y in self.array.year.values if y>=2015]
         ] = hem.get_emissions_per_powertrain("CNG", euro_class=6)
 
-        # Year index for EURO 5 pollution class, starts in 2009
+        # Year index for EURO 5 pollution class, starts in 2011, ends in 2014
         self.array.loc[
-            :, "ICEV-g", list_direct_emissions, [y for y in self.array.year.values if 2009 <= y < 2015]
+            :, "ICEV-g", list_direct_emissions, [y for y in self.array.year.values if 2011 <= y < 2015]
         ] = hem.get_emissions_per_powertrain("CNG", euro_class=5)
 
-        # Year index for EURO 4 pollution class, starts in 2005, ends in 2009
+        # Year index for EURO 4 pollution class, starts in 2006, ends in 2010
         self.array.loc[
-            :, "ICEV-g", list_direct_emissions, [y for y in self.array.year.values if 2005 <= y < 2009]
+            :, "ICEV-g", list_direct_emissions, [y for y in self.array.year.values if 2006 <= y < 2011]
         ] = hem.get_emissions_per_powertrain("CNG", euro_class=4)
 
-         # Year index for EURO 3 pollution class, starts in 2000, ends in 2004
+        # Year index for EURO 3 pollution class, starts in 2001, ends in 2005
         self.array.loc[
-            :, "ICEV-g", list_direct_emissions, [y for y in self.array.year.values if 2000 <= y < 2004]
+            :, "ICEV-g", list_direct_emissions, [y for y in self.array.year.values if 2001 <= y < 2006]
         ] = hem.get_emissions_per_powertrain("CNG", euro_class=3)
 
+        # Year index for EURO 2 pollution class, ends in 2000
+        self.array.loc[
+            :, "ICEV-g", list_direct_emissions, [y for y in self.array.year.values if y < 2001]
+        ] = hem.get_emissions_per_powertrain("CNG", euro_class=2)
+
         # Applies an emission factor, useful for sensitivity purpose
         self.array.loc[:, "ICEV-g", list_direct_emissions, :] *= self.array.loc[
             :, "ICEV-g", "emission factor", :
@@ -889,7 +927,7 @@ class CarModel:
             :, ["HEV-p", "HEV-d"], list_noise_emissions, :, :
         ] = nem.get_sound_power_per_compartment("hybrid").reshape((24, 1, 1))
 
-    def calculate_cost_impacts(self, sensitivity=False):
+    def calculate_cost_impacts(self, sensitivity=False, scope=None):
         """
         This method returns an array with cost values per vehicle-km, sub-divided into the following groups:
 
@@ -903,21 +941,33 @@ class CarModel:
         :rtype: xarray.core.dataarray.DataArray
         """
 
+        if scope is None:
+            scope = {}
+            scope["size"] = self.array.coords["size"].values.tolist()
+            scope["powertrain"] = self.array.coords["powertrain"].values.tolist()
+            scope["year"] = self.array.coords["year"].values.tolist()
+        else:
+            scope["size"] = scope.get("size", self.array.coords["size"].values.tolist())
+            scope["powertrain"] = scope.get(
+                "powertrain", self.array.coords["powertrain"].values.tolist()
+            )
+            scope["year"] = scope.get("year", self.array.coords["year"].values.tolist())
+
         list_cost_cat = ["purchase", "maintenance", "component replacement", "energy", "total"]
 
         response = xr.DataArray(
             np.zeros(
-                (len(self.array.coords['size']),
-                 len(self.array.coords['powertrain']),
+                (len(scope['size']),
+                 len(scope['powertrain']),
                  len(list_cost_cat),
-                 len(self.array.coords['year']),
-                 len(self.array.coords['value']))
+                 len(scope['year']),
+                 len(self.array.coords['value'].values))
             ),
             coords=[
-                self.array.coords['size'].values.tolist(),
-                self.array.coords['powertrain'].values.tolist(),
+                scope['size'],
+                scope['powertrain'],
                 ["purchase", "maintenance", "component replacement", "energy", "total"],
-                self.array.coords['year'].values.tolist(),
+                scope['year'],
                 self.array.coords['value'].values.tolist(),
             ],
             dims=["size", "powertrain", "cost_type", "year", "value"],
@@ -925,7 +975,9 @@ class CarModel:
 
         response.loc[
             :,:,["purchase", "maintenance", "component replacement", "energy", "total"],:,:
-        ] = self.array.sel(
+        ] = self.array.sel(powertrain=scope['powertrain'],
+                           size=scope['size'],
+                           year=scope['year'],
             parameter=["amortised purchase cost",
                        "maintenance cost",
                        "amortised component replacement cost",
